@@ -1,7 +1,7 @@
 package Chemistry::Atom;
 
-$VERSION = '0.24';
-# $Id: Atom.pm,v 1.25 2004/06/16 19:34:28 itubert Exp $
+$VERSION = '0.30';
+# $Id: Atom.pm,v 1.35 2004/11/10 00:00:43 itubert Exp $
 
 =head1 NAME
 
@@ -12,9 +12,9 @@ Chemistry::Atom - Chemical atoms as objects in molecules
     use Chemistry::Atom;
 
     my $atom = new Chemistry::Atom(
-	id => 'a1',
-	coords => [$x, $y, $z],
-	symbol => 'Br'
+        id => 'a1',
+        coords => [$x, $y, $z],
+        symbol => 'Br'
     );
 
     print $atom->print;
@@ -22,7 +22,7 @@ Chemistry::Atom - Chemical atoms as objects in molecules
 =head1 DESCRIPTION
 
 This module includes objects to describe chemical atoms. 
-An atom is defined by its symbol and its coordinates.
+An atom is defined by its symbol and its coordinates, among other attributes.
 Atomic coordinates are described by a Math::VectorReal
 object, so that they can be easily used in vector operations.
 
@@ -30,12 +30,18 @@ object, so that they can be easily used in vector operations.
 
 In addition to common attributes such as id, name, and type, 
 atoms have the following attributes, which are accessed or
-modified through methods defined below: bonds, coords, Z, symbol.
+modified through methods defined below: bonds, coords, internal_coords,
+Z, symbol, etc.
 
-In general, to get the value of a property use $atom->method without
-any parameters. To set the value, use $atom->method($new_value).
+In general, to get the value of a property, use $atom->method without
+any parameters. To set the value, use $atom->method($new_value). When setting
+an attribute, the accessor returns the atom object, so that accessors can be
+chained:
+
+    $atom->symbol("C")->name("CA")->coords(1,2,3);
 
 =cut
+
 # Considering to add the following attributes:
 # mass_number (A)
 
@@ -49,17 +55,15 @@ use Carp;
 use base qw(Chemistry::Obj Exporter);
 
 our @EXPORT_OK = qw(distance angle dihedral angle_deg dihedral_deg);
-our @EXPORT = ();
 our %EXPORT_TAGS = (
-      all  => [@EXPORT, @EXPORT_OK],
+      all  => \@EXPORT_OK,
 );
 
 
-use vars qw(@ELEMENTS %ELEMENTS);
 
 my $N = 0; # Number of atoms created so far, used to generate default IDs.
 
-@ELEMENTS = qw(
+our @ELEMENTS = qw(
     n
     H                                                                   He
     Li  Be                                          B   C   N   O   F   Ne
@@ -74,6 +78,7 @@ my $N = 0; # Number of atoms created so far, used to generate default IDs.
             Lr  Rf  Db  Sg  Bh  Hs  Mt  Ds  Uuu Uub Uut Uuq Uup Uuh Uus Uuo
 );
 
+our %ELEMENTS;
 for (my $i = 1; $i < @ELEMENTS; ++$i){
     $ELEMENTS{$ELEMENTS[$i]} = $i;
 }
@@ -116,8 +121,7 @@ my %Atomic_masses = (
 
 =item Chemistry::Atom->new(name => value, ...)
 
-Create a new Atom object with the specified attributes. Sensible defaults
-are used when possible.
+Create a new Atom object with the specified attributes.
 
 =cut
 
@@ -158,7 +162,8 @@ sub Z {
 
     if(@_) {
         $self->{symbol} = $ELEMENTS[$_[0]];
-        return $self->{Z} = $_[0];
+        $self->{Z} = $_[0];
+        return $self;
     } else {
         return $self->{Z};
     }
@@ -175,9 +180,10 @@ sub symbol {
     my $self = shift;
 
     if(@_) {
-	$_[0] =~ s/ //g;
+        $_[0] =~ s/ //g;
         $self->{Z} = $ELEMENTS{$_[0]};
-        return $self->{symbol} = $_[0];
+        $self->{symbol} = $_[0];
+        return $self;
     } else {
         return $self->{symbol};
     }
@@ -185,31 +191,62 @@ sub symbol {
 
 =item $atom->mass($new_mass)
 
-Sets and returns the atomic mass in atomic mass units. By default, relative
-atomic masses from the 1995 IUPAC recommendation are used. (Table stolen from
-the Chemistry::MolecularMass module by Maksim A. Khrapov).
+Sets and returns the atomic mass in atomic mass units. Any arbitrary mass may
+be set explicitly by using this method. However, if no mass is set explicitly
+and this method is called as an accessor, the return value is the following:
+
+1) If the mass number is undefined (see the mass_number method below), the
+relative atomic mass from the 1995 IUPAC recommendation is used. (Table stolen
+from the Chemistry::MolecularMass module by Maksim A.  Khrapov).
+
+2) If the mass number is defined and the L<Chemistry::Isotope> module is 
+available and it knows the mass for the isotope, the exact mass of the isotope
+is used; otherwise, the mass number is returned.
 
 =cut
 
 sub mass {
-    my ($self, $mass) = @_;
-    if(defined $mass) {
-        $self->{mass} = $mass;
+    my $self = shift;
+    if (@_) {
+        $self->{mass} = shift;
+        return $self;
     } else {
-        if (exists $self->{mass}) {
-            $mass = $self->{mass};
+        if (defined $self->{mass}) {
+            return $self->{mass};
+        } elsif (defined $self->{mass_number}) {
+            if (eval { require Chemistry::Isotope } and 
+                my $m = Chemistry::Isotope::isotope_mass(
+                    $self->{mass_number}, $self->{Z})
+            ) {
+                return $m;
+            } else {
+                return $self->{mass_number};
+            }
         } else {
-            $mass = $Atomic_masses{$self->symbol};
+            return $Atomic_masses{$self->symbol};
         }
     }
-    $mass;
 }
 
-=item $atom->coords([$x, $y, $z])
+=item $atom->mass_number($new_mass_number)
 
-Sets the atom's coordinates, and returns a Math::VectorReal object.
-It can take as a parameter a Math::VectorReal object, a reference to an 
-array, or the list of coordinates.
+Sets or gets the mass number. The mass number is undefined unless is 
+set explicitly (this module does not try to guess a default mass number based
+on the natural occuring isotope distribution).
+
+=cut
+
+Chemistry::Obj::accessor('mass_number');
+
+=item $atom->coords
+
+    my $vector = $atom->coords;  # get a Math::VectorReal object
+    $atom->coords($vector);      # set a Math::VectorReal object 
+    $atom->coords([$x, $y, $z]); # also accepts array refs 
+    $atom->coords($x, $y, $z);   # also accepts lists
+
+Sets or gets the atom's coordinates. It can take as a parameter a
+Math::VectorReal object, a reference to an array, or the list of coordinates.
 
 =cut
 
@@ -218,21 +255,65 @@ sub coords {
 
     if(@_) {
         if (UNIVERSAL::isa($_[0], "Math::VectorReal")) {
-            return $self->{coords} = $_[0];
+            $self->{coords} = $_[0];
         } elsif (ref $_[0] eq "ARRAY") {
-            return $self->{coords} = vector(@{$_[0]});
+            $self->{coords} = vector(@{$_[0]});
         } else {
-            return $self->{coords} = vector(@_);
+            $self->{coords} = vector(@_);
         }
     } else {
         return $self->{coords};
     }
+    $self;
+}
+
+=item $atom->internal_coords
+
+    # get a Chemistry::InternalCoords object
+    my $ic = $atom->internal_coords;      
+
+    # set a Chemistry::InternalCoords object 
+    $atom->internal_coords($vic);         
+
+    # also accepts array refs 
+    $atom->internal_coords([8, 1.54, 7, 109.47, 6, 120.0]);   
+
+    # also accepts lists
+    $atom->internal_coords(8, 1.54, 7, 109.47, 6, 120.0);    
+
+Sets or gets the atom's internal coordinates. It can take as a parameter a
+Chemistry::InternalCoords object, a reference to an array, or the list of
+coordinates. In the last two cases, the list elements are the following: atom 
+number or reference for distance, distance, atom number or reference for angle,
+angle in degrees, atom number or reference for dihedral, dihedral in degrees.
+
+=cut
+
+sub internal_coords {
+    my $self = shift;
+
+    if(@_) {
+        if (UNIVERSAL::isa($_[0], "Chemistry::InternalCoords")) {
+            $self->{internal_coords} = $_[0];
+        } elsif (ref $_[0] eq "ARRAY") {
+            require Chemistry::InternalCoords;
+            $self->{internal_coords} = 
+                Chemistry::InternalCoords->new($self, @{$_[0]});
+        } else {
+            require Chemistry::InternalCoords;
+            $self->{internal_coords} = 
+                Chemistry::InternalCoords->new($self, @_);
+        }
+    } else {
+        return $self->{internal_coords};
+    }
+    $self;
 }
 
 =item $atom->x3, $atom->y3, $atom->z3
 
 Get the x, y or z 3D coordinate of the atom. This methods are just accessors
-that don't change the coordinats. $atom->x3 is short for 
+that don't change the coordinates. $atom->x3 is short for 
 ($atom->coords->array)[0], and so on.
 
 =cut
@@ -247,54 +328,84 @@ Set or get the formal charge of the atom.
 
 =cut
 
-sub formal_charge {
-    my $self = shift;
-    if (@_) {
-        ($self->{formal_charge}) = @_;
-        return $self;
-    } else {
-        return $self->{formal_charge};
-    }
-}
+Chemistry::Obj::accessor('formal_charge');
 
-=item $atom->hydrogens($h_count)
+=item $atom->implicit_hydrogens($h_count)
 
 Set or get the number of implicit hydrogen atoms bonded to the atom.
 
 =cut
 
-sub hydrogens {
-    my $self = shift;
-    if (@_) {
-        ($self->{hydrogens}) = @_;
-        return $self;
-    } else {
-        return $self->{hydrogens};
+sub implicit_hydrogens { shift->hydrogens(@_) }
+
+=item $atom->hydrogens($h_count)
+
+Set or get the number of implicit hydrogen atoms bonded to the atom
+(DEPRECATED: USE C<implicit_hydrogens> INSTEAD).
+
+=cut
+
+Chemistry::Obj::accessor('hydrogens');
+
+=item $atom->total_hydrogens($h_count)
+
+Get the total number of hydrogen atoms bonded to the atom (implicit +
+explicit).
+
+=cut
+
+sub total_hydrogens {
+    my ($self) = @_;
+    no warnings 'uninitialized';
+    $self->hydrogens + grep { $_->symbol eq 'H' } $self->neighbors;
+}
+
+=item $atom->sprout_hydrogens
+
+Convert all the implicit hydrogens for this atom to explicit hydrogens. Note:
+it does B<not> generate coordinates for the new atoms.
+
+=cut
+
+sub sprout_hydrogens {
+    my ($self) = @_;
+    for (1 .. $self->implicit_hydrogens) {
+        $self->parent->new_bond(
+            atoms => [$self, $self->parent->new_atom(symbol => 'H')]);
     }
+    $self->implicit_hydrogens(0);
+}
+
+=item $atom->collapse_hydrogens
+
+Delete neighboring hydrogen atoms and add them as implicit hydrogens for this
+atom.
+
+=cut
+
+sub collapse_hydrogens {
+    my ($self) = @_;
+    my $implicit = $self->implicit_hydrogens;
+    for my $nei ($self->neighbors) {
+        $nei->delete, $implicit++ if $nei->symbol eq 'H';
+    }
+    $self->implicit_hydrogens($implicit);
 }
 
 =item $atom->aromatic($bool)
 
 Set or get whether the atom is considered to be aromatic. This property may be
 set arbitrarily, it doesn't imply any kind of "intelligent aromaticity
-detection"!
+detection"! (For that, look at the L<Chemistry::Ring> module).
 
 =cut
 
-sub aromatic {
-    my $self = shift;
-    if (@_) {
-        ($self->{aromatic}) = @_;
-        return $self;
-    } else {
-        return $self->{aromatic};
-    }
-}
+Chemistry::Obj::accessor('aromatic');
 
 =item $atom->valence
 
 Returns the sum of the bond orders of the bonds in which the atom participates,
-including implicit hydrogens.
+including implicit hydrogens (which are assumed to have bond orders of one).
 
 =cut
 
@@ -306,12 +417,21 @@ sub valence {
     $valence;
 }
 
-=item $atom->add_bond($bond)
+=item $atom->explicit_valence
 
-Adds a new bond to the atom, as defined by the Bond object $bond.
+Like C<valence>, but excluding implicit hydrogen atoms. To get the raw number
+of bonds, without counting bond orders, call $atom->bonds in scalar context.
 
 =cut
 
+sub explicit_valence {
+    my ($self) = @_;
+    my $valence = 0;
+    $valence += $_->order for $self->bonds;
+    $valence;
+}
+
+# this method is for internal use only; called by $bond->atoms
 sub add_bond {
     my $self = shift;
     my $bond = shift;
@@ -346,16 +466,22 @@ sub delete_bond {
 
 =item $atom->delete
 
-Calls $mol->delete_atom($atom) on the atom's parent molecule. Note that an atom
-should belong to only one molecule or strange things may happen.
+Calls $mol->delete_atom($atom) on the atom's parent molecule.
 
 =cut
 
 sub delete {
     my ($self) = @_;
-    $self->{parent}->delete_atom($self);
+    $self->{parent}->_delete_atom($self);
 }
 
+=item $atom->parent
+
+Returns the atom's containing object (the molecule to which the atom belongs).
+An atom can only have one parent.
+
+=cut
+
 sub parent {
     my $self = shift;
     if (@_) {
@@ -367,10 +493,11 @@ sub parent {
     }
 }
 
-=item $atom->neighbors([$from])
+=item $atom->neighbors($from)
 
 Return a list of neighbors. If an atom object $from is specified, it will be
-excluded from the list.
+excluded from the list (this is useful if an atom wants to know who its 
+neighbor's neighbors are, without counting itself).
 
 =cut
 
@@ -380,15 +507,15 @@ sub neighbors {
     my @ret = ();
 
     for my $b (@{$self->{bonds}}) {
-	push @ret, $b->{to} unless $from && $b->{to} eq $from;
+        push @ret, $b->{to} unless $from && $b->{to} eq $from;
     }
     @ret;
 }
 
-=item $atom->bonds([$from])
+=item $atom->bonds($from)
 
-Return a list of bonds. If an atom object $from is specified, it will be
-excluded from the list.
+Return a list of bonds. If an atom object $from is specified, bonds to
+that atom will be excluded from the list.
 
 =cut
 
@@ -398,17 +525,21 @@ sub bonds {
     my @ret = ();
 
     for my $b (@{$self->{bonds}}) {
-	push @ret, $b->{bond} unless $from && $b->{to} eq $from;
+        push @ret, $b->{bond} unless $from && $b->{to} eq $from;
     }
     @ret;
 }
 
-=item $atom->bonds_neighbors([$from])
+=item $atom->bonds_neighbors($from)
 
 Return a list of hash references, representing the bonds and neighbors from the
 atom. If an atom object $from is specified, it will be excluded from the list.
 The elements of the hash are 'to', and atom reference, and 'bond', a bond
-reference.
+reference. For example, 
+
+    for my $bn ($atom->bonds_neighbors) {
+        print "bond $bn->{bond} point to atom $bn->{to}\n";
+    }
 
 =cut
 
@@ -418,7 +549,7 @@ sub bonds_neighbors {
     my @ret = ();
 
     for my $b (@{$self->{bonds}}) {
-	push @ret, {%$b} unless $from && $b->{to} eq $from;
+        push @ret, {%$b} unless $from && $b->{to} eq $from;
     }
     @ret;
 }
@@ -427,7 +558,8 @@ sub bonds_neighbors {
 
 Returns the minimum distance to $obj, which can be an atom, a molecule, or a
 vector. In scalar context it returns only the distance; in list context it
-also returns the closest atom found.
+also returns the closest atom found. It can also be called as a function,
+Chemistry::Atom::distance (which can be exported).
 
 =cut
 
@@ -461,7 +593,8 @@ sub distance {
 =item $atom->angle($atom2, $atom3)
 
 Returns the angle in radians between the atoms involved. $atom2 is the atom in
-the middle. Can also be called as Chemistry::Atom::angle($atom1, $atom2, $atom3);
+the middle. Can also be called as Chemistry::Atom::angle($atom1, $atom2,
+$atom3). This function can be exported.
 
 =cut
 
@@ -470,6 +603,7 @@ sub angle {
     @_ == 3 or croak "Chemistry::Atom::angle requires three atoms!\n";
     my @c;
     for my $a (@_) { # extract coordinates
+        ref $a or croak "Chemistry::Atom::angle: $a is not an object";
         push @c, $a->isa("Chemistry::Atom") ? $a->coords :
             $a->isa("Math::VectorReal") ? $a : 
                 croak "angle: $a is neither an atom nor a vector!\n";
@@ -481,18 +615,19 @@ sub angle {
 
 =item $atom->angle_deg($atom2, $atom3)
 
-Same as angle(), but returns the value in degrees.
+Same as angle(), but returns the value in degrees. May be exported.
 
 =cut
 
 sub angle_deg {
-    rad2deg(angle(@_));
+    rad2deg(shift->angle(@_));
 }
 
 =item $atom->dihedral($atom2, $atom3, $atom4)
 
 Returns the dihedral angle in radians between the atoms involved.  Can also be
-called as Chemistry::Atom::dihedral($atom1, $atom2, $atom3, $atom4);
+called as Chemistry::Atom::dihedral($atom1, $atom2, $atom3, $atom4). May be
+exported.
 
 =cut
 
@@ -515,17 +650,17 @@ sub dihedral {
 
 =item $atom->dihedral_deg($atom2, $atom3, $atom4)
 
-Same as dihedral(), but returns the value in degrees.
+Same as dihedral(), but returns the value in degrees. May be exported.
 
 =cut
 
 sub dihedral_deg {
-    rad2deg(dihedral(@_));
+    rad2deg(shift->dihedral(@_));
 }
 
 =item $atom->print
 
-Convert the atom to a string representation.
+Convert the atom to a string representation (used for debugging).
 
 =cut
 
@@ -605,7 +740,7 @@ sub sprintf {
 =item $atom->printf($format)
 
 Same as $atom->sprintf, but prints to standard output automatically. Used
-for quick and dirty molecular information dumping.
+for quick and dirty atomic information dumping.
 
 =cut
 
@@ -620,23 +755,24 @@ sub printf {
 
 =head1 VERSION
 
-0.24
+0.30
 
 =head1 SEE ALSO
 
 L<Chemistry::Mol>, L<Chemistry::Bond>, 
-L<Math::VectorReal>, L<Chemistry::Tutorial>
+L<Math::VectorReal>, L<Chemistry::Tutorial>,
+L<Chemistry::InternalCoords>
 
 The PerlMol website L<http://www.perlmol.org/>
 
 =head1 AUTHOR
 
-Ivan Tubert E<lt>itub@cpan.orgE<gt>
+Ivan Tubert-Brohman E<lt>itub@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2004 Ivan Tubert. All rights reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms as
+Copyright (c) 2004 Ivan Tubert-Brohman. All rights reserved. This program is
+free software; you can redistribute it and/or modify it under the same terms as
 Perl itself.
 
 =cut
